#!/usr/bin/perl -I perl

# $Id$

=pod
=head1 Glintfreq.pl

Glintfreq.p - This program runs Primer3 and parses the output File

=head1  SYNOPSIS

glintfreqFile

=head1 DESCRIPTION

This program create a BoulderIO file from a multifasta file,
then runs Primer 3 using a system commandline
and finally it parses the outputfile and return all the informations 
on the bunch of primers

=cut

=head1 SUBROUTINES

=cut

package Copy;

use strict;
use warnings;

use base qw( LipmObject);
use Region;
use SuitableRegion;
use Probe;

############################################################
###                   CONSTRUCTEURS                      ###
############################################################

=head2 function _Init

 Title    : _Init
 Usage    : The constructor
 Access   : Private
 Function : Set all arguments
            Push the new copy in the copy array of the probe
 Args     : $probeId => the probe id
            $seqId   => the sequence id
            $strand  => the strand of the copy
            $from    => the start position of the copy
            $to      => the end position of the copy
            $mism    => number of mismatch 
            $gaps    => number of gap
            $actLen  => the lenght of the primer find
            $expLen  => the length of the primer expected

 Returns  : The object
 Globals  : none

=cut

sub _Init
{
	my $self = shift;
	
	my ( $probeId, $seqId, $strand, $from, $to, $mism, $gaps, $actLen, $expLen, $ro_region ) = @_ ;
	
	$self ->_Set ( '__probeId', $probeId );
	$self ->_Set ( '__seqId', $seqId );
	$self ->_Set ( '__strand', $strand );
	$self ->_Set ( '__from', $from );
	$self ->_Set ( '__to', $to );
	$self ->_Set ( '__mism', $mism );
	$self ->_Set ( '__gaps', $gaps );
	$self ->_Set ( '__actLen', $actLen );
	$self ->_Set ( '__expLen', $expLen );
	
	if ( defined $probeId ) { $$ro_region->PushCopy($probeId, $self); }
	
	return $self;
}

############################################################
###                   GETTERS/SETTERS                    ###
############################################################


=head2 function From

 Title        : From
 Usage        : $from = $probes->From();
 Function     : Accessor, returns the copy start position
 Access       : Public
 Returns      : $from
 Args         : none

=cut
sub From
{
	my $self = shift;
	return $self->_Get('__from');
}
1;
