#!/usr/bin/perl -I perl

# $Id$

=pod
=head1 Glintfreq.pl

Glintfreq.p - This program runs Primer3 and parses the output File

=head1  SYNOPSIS

glintfreqFile

=head1 DESCRIPTION

This program create a BoulderIO file from a multifasta file,
then runs Primer 3 using a system commandline
and finally it parses the outputfile and return all the informations 
on the bunch of primers

=cut
=head1 SUBROUTINES

GetAProbe
PushProbe

GeneratePrimer3InputFile
LaunchPrimer3
LoadProbe

=cut

package SuitableRegion;

use strict;
use warnings;

use base qw( Region);

use FindBin qw( $Bin );

############################################################
###                   CONSTRUCTEURS                      ###
############################################################

=head2 function _Init

 Title    : _Init
 Usage    : The constructor
 Access   : Private
 Function : Set all arguments and create the probe array
            Push the new suitable subregion in the suitable array of region
 Args     : $id        => the subregion id
            $offset    => the subregion offset
            $size      => the subregion size
            $ra_probes => the probes reference array
            $coverage  => the coverage of suitableRegion by valid and non overlap probe
 Returns  : The object
 Globals  : none

=cut
sub _Init
{
	my ( $self, $id, $offset, $from, $size ) = @_ ;
	
	my @ao_probes = ();
	
	$self ->_Set ( '__id'    , $id     );
	$self ->_Set ( '__offset', $offset );
	$self ->_Set ( '__from'  , $from   );
	$self ->_Set ( '__size'  , $size   );
	$self ->_Set ( '__probes', \@ao_probes );
	
	my $coverage = undef;
	$self ->_Set ( '__coverage', $coverage );
	
	return $self;
}

############################################################
###                   GETTERS/SETTERS                    ###
############################################################

=head2 function Start

 Title        : Start
 Usage        : $start = $suitable->Start();
 Function     : Accessor, returns the start position
 Access       : Public
 Returns      : $regionStart
 Args         : none

=cut
sub Start
{
	my $self        = shift;
	my $offset      = $self->_Get('__offset');
	my $ro_region   = $self->_Get('__region');
	my $regionStart = $$ro_region->Start();
	
	return ($regionStart+$offset);
}

=head2 function Id

 Title        : Id
 Usage        : $id = $suitable->Id();
 Function     : Accessor, returns the suitable subregion id
 Access       : Public
 Returns      : $id
 Args         : none

=cut
sub Id
{
	my $self = shift;
	return $self->_Get('__id');
}

=head2 function Offset

 Title        : Offset
 Usage        : $offset = $suitable->Offset();
 Function     : Accessor, returns the subregion offset
 Access       : Public
 Returns      : $offset
 Args         : none

=cut
sub Offset
{
	my $self = shift;
	return $self->_Get('__offset');
}

=head2 function From

 Title        : From
 Usage        : $from = $suitable->From();
 Function     : Accessor, returns the subregion start position
 Access       : Public
 Returns      : $from
 Args         : none

=cut
sub From
{
	my $self = shift;
	return $self->_Get('__from');
}

=head2 function Size

 Title        : Size
 Usage        : $size = $suitable->Size();
 Function     : Accessor, returns the subregion size
 Access       : Public
 Returns      : $size
 Args         : none

=cut
sub Size
{
	my $self = shift;
	return $self->_Get('__size');
}

=head2 function Probes

 Title        : Probes
 Usage        : $ra_probes = $SuitableRegion->Probes();
 Function     : Accessor, returns the probe array reference 
 Access       : Public
 Returns      : $ra_probes
 Args         : none
 
=cut
sub Probes
{
	my $self = shift;
	my $rao_probes = $self->_Get('__probes');
	return $rao_probes;
}

=head2 function Coverage

 Title        : Coverage
 Usage        : $coverage = $SuitableRegion->Coverage();
 Function     : Accessor, returns the coverage of the suitable region
 Access       : Public
 Returns      : $coverage
 Args         : none
 
=cut
sub Coverage
{
	my $self = shift;
	my $coverage = $self->_Get('__coverage');
	return $coverage;
}

=head2 Procedure _SetCoverage

 Title        : _SetCoverage
 Usage        : $suitable->_SetCoverage( $coverage );
 Prerequisite : none
 Procedure    : none
 Args         : $coverage

=cut
sub SetCoverage
{
	my ( $self, $coverage ) = @_ ;
	$self ->_Set( '__coverage', $coverage );
}

=head2 function GetAProbe

 Title        : GetAProbe
 Usage        : $probe = $suitable->GetAProbe($id);
 Function     : Accessor, returns the probe corresponding to the $id 
 Access       : Public
 Returns      : $probe
 Args         : none
 
=cut
sub GetAProbe
{
	my ( $self, $id ) = @_;
	
	my $suitableId = int($id/10);
	my $probeId = $id-($suitableId*10);
	
	my $ra_probes = $self->_Get('__probes');
	
	return $$ra_probes[$probeId];
}

############################################################
###                     FUNCTION                         ###
############################################################

=head2 function GetTempFileName

 Title        : GetTempFileName
 Usage        : $tempFileName = &GetTempFileName($process);
 Function     : Accessor, returns the Amplicon start position
 Access       : Public
 Returns      : $from
 Args         : none

=cut
sub GetTempFileName
{
	my ( $processus, $type, $ext ) = @_;
	
	unless ( defined $ext ) { $ext = '' ; }
	die "\n Error : Wrong argument in GetTempFileName $!\n" if ( $type !~ /(in|out|log|param)/ );
	
	my $tempFileName = $processus.'.'.$type.'.GF-tmp'.$ext;
	
	return $tempFileName;
}

=head2 function PushProbe

 Title        : PushProbe
 Usage        : $suitable->PushProbe($probe);
 Function     : Push a probe in the probe array of the suitable object
 Access       : Public
 Returns      : none
 Args         : the probe pushed

=cut
sub PushProbe
{
	my ( $self, $o_probe ) = @_;
	my $rao_probes = $self->_Get('__probes');
	push(@$rao_probes, $o_probe);
}

=head2 function ExtractSubregionSequence

 Title        : ExtractSubregionSequence
 Usage        : $suitable->ExtractSubregionSequence($file_name, $offset, $size);
 Function     : Using Bioperl library, extract a subregion sequence and return it as a string
 Access       : Public
 Returns      : subregion sequence string
 Args         : the source file, the offset and the size of the subregion

=cut
sub ExtractSubregionSequence
{
	my ( $self, $file, $offset, $end ) = @_;
	
	my $seq = Bio::SeqIO->new( -file => $file, -format => 'fasta' );
	my $seqObj = $seq->next_seq();
	my $subseq = $seqObj->subseq( $offset, $end );
	
	return ( $subseq );
}

###### PRIMER3 ######

=head2 function GeneratePrimer3InputFile

 Title        : GeneratePrimer3InputFile
 Usage        : $subregion->GeneratePrimer3InputFile();
 Function     : Create the Primer3 input file using the subregion object
 Access       : Public
 Returns      : none
 Args         : none
 
=cut
sub GeneratePrimer3InputFile
{
	my ( $self, $query_Filename ) = @_;
	
	my $input_Filename = &GetTempFileName( 'Primer3', 'in' );
	
	open( FILE, ">$input_Filename" ) || die "Erreur E/S:$!\n";
	
		my $id     = $self->Id();
		my $from   = $self->From();
		my $size   = $self->Size();
		my $offset = $self->Offset();
		my $end    = $offset+$size;
		
		print FILE "SEQUENCE_ID=$id|\tstart : $from\tend : $end\n";
		my $sequence = $self->ExtractSubregionSequence( $query_Filename, $offset, $end );
		print FILE "SEQUENCE_TEMPLATE=$sequence\n=\n";
	
	close ( FILE );
}

=head2 function LaunchPrimer3

 Title        : LaunchPrimer3
 Usage        : &LaunchPrimer3();
 Function     : Run Primer3 software
 Access       : Public
 Returns      : none
 Args         : none
 
=cut
sub LaunchPrimer3
{
	my ( $self, $primer3_exe ) = @_ ;
	
	my $primerInputFile  = &GetTempFileName( 'Primer3', 'in'   );
	my $primerOutputFile = &GetTempFileName( 'Primer3', 'out'  );
	my $primerParamFile  = &GetTempFileName( 'Primer3', 'param');
	
	my $command = $primer3_exe." -p3_settings_file $primerParamFile <$primerInputFile -output $primerOutputFile";
	system ( $command ) ;
}

=head2 function LoadProbe

 Title        : LoadProbe
 Usage        : $subregion->LoadProbe();
 Function     : Parses the Primer3 output
                and Load Probes in the region object
 Access       : Public
 Returns      : none
 Args         : none
 
=cut
sub LoadProbe
{
	my $self = shift;
	
	my $output_file = &GetTempFileName( 'Primer3', 'out' );

	#Get all the lines of the file
	open ( FILE, $output_file ) || die "\nPrimer3 :\nErreur E/S : $output_file $!\n";
		my $all_lines = '';
		my $line      = undef;
		while( $line = <FILE> ) { $all_lines .= $line ; }
	close ( FILE );
	
	#Split the different sequences with '='
	my @a_line  = split ( /\n/, $all_lines ) ;
	
	my $seq     = undef ;
	my $nb_pair = undef ;
	my $id      = undef ;
	my $end     = undef ;
	my $suitable_from = undef;

	if ( $a_line[0] =~ /SEQUENCE_ID=(\d+)\|\tstart : (\d+)\tend : (\d+)/) 
	{
		$id  = $1;
		$end = $3;
		$suitable_from = $2;
	}
	if ( $a_line[1] =~ /SEQUENCE_TEMPLATE=(.*)/        ) { $seq = $1 ;     }
	if ( $a_line[5] =~ /PRIMER_PAIR_NUM_RETURNED=(.*)/ ) { $nb_pair = $1 ; }
	
	for ( my $i = 6 ; $i < $#a_line ; $i += 20 ) 
	{
		my $pair                = undef ;
		my $pair_id             = undef ;
		my $pair_penalty        = undef ;
		my $primer_right_seq    = undef ;
		my $primer_left_seq     = undef ;
		my $offset              = undef ;
		my $primer_left_length  = undef ;
		my $primer_right_start  = undef ;
		my $primer_right_length = undef ;
		my $primer_right_tm     = undef ;
		my $primer_left_tm      = undef ;
		my $primer_right_gc     = undef ;
		my $primer_left_gc      = undef ;
		my $size                = undef ;
		
		for ( my $j = $i ; $j <= $i + 19 ; $j++ )
		{
			if ( $a_line[$j] =~ /PRIMER_PAIR_(\d+)_PENALTY=(.*)/ )
			{
				$pair_id = $1 ;
				$pair_penalty = $2 ;
			}
			if ( $a_line[$j] =~ /PRIMER_LEFT_\d+_SEQUENCE=(.*)/  ) { $primer_left_seq = $1 ;  }
			if ( $a_line[$j] =~ /PRIMER_RIGHT_\d+_SEQUENCE=(.*)/ ) { $primer_right_seq = $1 ; }
			if ( $a_line[$j] =~ /PRIMER_LEFT_\d+=(\d+),(\d+)/ )
			{
				$offset = $1 ;
				$primer_left_length = $2 ;
			}
			if ( $a_line[$j] =~ /PRIMER_RIGHT_\d+=(\d+),(\d+)/ )
			{
				$primer_right_start = $1 ;
				$primer_right_length = $2 ;
			}
			if ( $a_line[$j] =~ /PRIMER_LEFT_\d+_TM=(.*)/   ) { $primer_left_tm= $1 ;   }
			if ( $a_line[$j] =~ /PRIMER_RIGHT_\d+_TM=(.*)/ ) { $primer_right_tm = $1 ; }

			if ( $a_line[$j] =~ /PRIMER_RIGHT_\d+_GC_PERCENT=(.*)/  ) { $primer_right_gc = $1 ; }
			if ( $a_line[$j] =~ /PRIMER_LEFT_\d+_GC_PERCENT=(.*)/  ) { $primer_left_gc = $1 ;  }
			if ( $a_line[$j] =~ /PRIMER_PAIR_\d+_PRODUCT_SIZE=(.*)/ ) { $size = $1 ;            }
		}
		my $probe_id = ($id*10)+$pair_id;
		my $from = $suitable_from + $offset ;
		
		my $o_probe = New Probe($probe_id, $primer_left_seq, $primer_right_seq, $offset, $size, $id, $from , $pair_penalty, $primer_left_length, $primer_right_start, $primer_right_length, $primer_left_tm, $primer_right_tm, $primer_left_gc, $primer_right_gc);
		$self->PushProbe( $o_probe );
	}
}

1;
