package iANT::Graphics::Display;

# le nom n'est sans doute pas bien choisi, faut 'il typer plus vers la squence ?
# ca serait bien de faire un mode wrapline qui permettrait un retour a la ligne automatique (cf DrawMap)

use strict;
require 'UNIVERSAL.pl';

use iANT::Graphics::ColorMap;
use iANT::Graphics::Scale;
use iANT::DB::Feature;
use iANT::Entry;
use iANT::Graphics::Segment;
use iANT::Graphics::Label;

use Carp;
use GD;

use constant DO_NOT_INCLUDE_MISSING_REGIONS => 1;
use constant INCLUDE_MISSING_REGIONS        => 1;

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 1 $ =~ /\d+/g); $r[0]};
}

=item New

=cut

sub _Init
{
    my ($self, %r_param) = @_;

    my $width          = $r_param{'width'};
    my $height         = $r_param{'width'};
    my $len            = $r_param{'len'};
    my $cm_type        = $r_param{'cm_type'};
    my $display_format = $r_param{'format'};

    $cm_type        = 'RBG'  if (!defined($cm_type));
    $display_format = 'HTML' if (!defined($display_format));

    if (!defined($width) || !defined($height) || (($width * $height) <= 0))
    {
        Carp::croak("Fatal error in constructor: width=$width, height=$height");
    }
    Carp::croak("Fatal error in construstor: the length of the sequence must be specified\n") if (!defined($len));

    $self->{'__o_scale'}       = '@';
    $self->{'__width'}         = $width;
    $self->{'__height'}        = $height;
    $self->{'__len'}           = $len;
    $self->{'__format'}        = $display_format;         # html, biomoby
    $self->{'__verbose'}       = 0;
    $self->{'__margin_left'}   = 0;
    $self->{'__margin_right'}  = 0;
    $self->{'__margin_top'}    = 0;
    $self->{'__margin_bottom'} = 0;
    $self->{'__scaling'}       = 1;
    $self->{'__vstep'}         = 20;
    $self->{'__map'}           = "";
    $self->{'__mode'}          = "nowrap";
    $self->{'__file_img'}      = "/tmp/Display.$$.img";
    $self->{'__file_map'}      = "/tmp/Display.$$.map";
    $self->{'__file_fmt'}      = "png";                   # format 'png', 'gif'
    $self->{'__h_colors'}      = {};
    $self->{'__ao_segments'}   = undef;
    $self->{'__ao_labels'}     = undef;
    $self->{'__o_gd'}          = '@';                     # gdimage;
    $self->{'__o_cm'}          = '@';                     # colormap

    $self->{'__o_scale'} = New iANT::Graphics::Scale();
    $self->{'__o_cm'}    = New iANT::Graphics::ColorMap($cm_type);    # $cm_type = 'RBG'
    $self->{'__o_gd'}    = new GD::Image($width, $height);

    $self->__LoadColorMap();
    $self->__ComputeScaling();
}

=head3 Load

 Title        : Load
 Usage        : Load

=cut

sub Load
{
    my ($self, $draw_missing_region, $type, $id_pattern, $rao_features) = @_;

    foreach my $o_feature (@{$rao_features})
    {
        my $o_hit        = $o_feature->GetHit();
        my $o_sequence   = $o_feature->GetSequence();
        my $sbegin       = $o_sequence->Get('begin');
        my $send         = $o_sequence->Get('end');
        my $slen         = $o_sequence->Get('length');
        my $target_id    = $o_hit->GetId();
        my $target_acc   = $o_hit->Get('accession');
        my $target_begin = $o_hit->Get('begin');
        my $target_end   = $o_hit->Get('end');
        my $target_desc  =
          "$sbegin-$send " . $o_hit->Get('accession') . "($target_begin-$target_end) " . $o_hit->Get('definition');
        my $target_score = abs($o_hit->Get('begin') - $o_hit->Get('end')) + 1;
        my ($item_id) = $o_hit->Get('accession') =~ /$id_pattern/;
        my $o_segment =
          New iANT::Graphics::Segment(
                                      'id'          => $item_id,
                                      'acc'         => $o_hit->Get('accession'),
                                      'begin'       => $sbegin,
                                      'end'         => $send,
                                      'score'       => $target_score,
                                      'type'        => $type,
                                      'description' => $target_desc,
                                      'color'       => $self->{'__h_colors'}{'blue'},
                                      'priority'    => 10
                                      );
        push(@{$self->{'__ao_segments'}}, $o_segment);

        if ($draw_missing_region)
        {

            # left
            my $delta = $target_begin - 1;
            if ($delta > 0)
            {
                my $left_begin = $sbegin - $delta;
                $left_begin = 1 if ($left_begin < 1);
                my $o_segment =
                  New iANT::Graphics::Segment(
                                              'id'          => $item_id,
                                              'acc'         => $o_hit->Get('accession'),
                                              'begin'       => $left_begin,
                                              'end'         => $sbegin - 1,
                                              'score'       => $target_score,
                                              'type'        => $type,
                                              'description' => "",
                                              'color'       => $self->{'__h_colors'}{'lightgrey'},
                                              'priority'    => 0
                                              );
                push(@{$self->{'__ao_segments'}}, $o_segment);
            }

            # right
            $delta = $o_hit->Get('length') - $target_end;
            if ($delta > 0)
            {
                my $right_end = $send + $delta;
                $right_end = $slen if ($right_end > $slen);
                my $o_segment =
                  New iANT::Graphics::Segment(
                                              'id'          => $item_id,
                                              'acc'         => $o_hit->Get('accession'),
                                              'begin'       => $send + 1,
                                              'end'         => $right_end,
                                              'score'       => $target_score,
                                              'type'        => $type,
                                              'description' => "",
                                              'color'       => $self->{'__h_colors'}{'lightgrey'},
                                              'priority'    => 0
                                              );
                push(@{$self->{'__ao_segments'}}, $o_segment);
            }
        }
        my $o_label = New iANT::Graphics::Label($item_id, $target_desc, $self->{'__h_colors'}{'black'});
        push(@{$self->{'__ao_labels'}}, $o_label);
    }
}

=head3 DrawHits

vue standard/horizontale d'un hit

=cut

sub DrawHits
{
    my ($self, %r_opt) = @_;

    #my ($self,%r_opt$sortmethod,$resizeheight) = @_;
    my $track         = $self->{'__margin_top'};
    my %h_memory_line = ();                        # a dfinir commme variable de classe sans doute
    my %h_memory_acc  = ();

    print STDERR "resize=" . $r_opt{'resize'} . " / sortbypos=" . $r_opt{'sortbypos'} . "\n";

    my ($limit_left, $limit_top, $limit_right, $limit_bottom) = $self->GetPxLimits();
    print STDERR "($limit_left,$limit_top,$limit_right,$limit_bottom) " . $#{$self->{'__ao_segments'}} . "\n";

    foreach my $o_segment (
        sort {
            if ($r_opt{'sortbypos'}) {return $a->{'__begin'} <=> $b->{'__begin'}}
            $b->{'__score'} <=> $a->{'__score'};

        } @{$self->{'__ao_segments'}}
      )
    {
        next if ($o_segment->{'__priority'} == 0);    # je ne tiens pas compte des rgions manquantes
        my $id = $o_segment->GetId();
        if (!defined($h_memory_line{$id}))
        {
            $track += $self->{'__vstep'};
            $h_memory_line{$id} = $track;
        }
    }
    if ($r_opt{'resize'})
    {
        $self->__Resize($self->{'__width'}, $track + $self->{'__margin_bottom'} + 20);
        ($limit_left, $limit_top, $limit_right, $limit_bottom) = $self->GetPxLimits();
    }

    foreach my $o_segment (
        sort {

            #if ( defined($sortmethod) && $sortmethod eq 'sortbypos' )
            if ($r_opt{'sortbypos'})
            {
                if ($a->{'__priority'} == $b->{'__priority'}) {return $a->{'__begin'} <=> $b->{'__begin'}}
                else {return $b->{'__priority'} <=> $a->{'__priority'}}
            }
            else
            {
                if ($a->{'__score'} == $b->{'__score'}) {return $a->{'__priority'} <=> $b->{'__priority'}}
                else {return $b->{'__score'} <=> $a->{'__score'}}
            }
        } @{$self->{'__ao_segments'}}
      )
    {
        if ($self->{'__mode'} eq 'wrap')    # c'est DisplayFam
        {
            print STDERR "Not implemented yet\n";
            next;
        }
        else
        {
            my $id = $o_segment->GetId();

            my $ret =
              $o_segment->DrawNoWrap($self->{'__margin_left'},
                                     $limit_right, $h_memory_line{$id}, $self->GetScaling(), $self->{'__o_gd'});
            if ($ret == 0)
            {

                # le dessin est impossible
            }
            elsif ($ret == -1)
            {

                # dessin partiel
            }
        }

    }
    foreach my $o_label (@{$self->{'__ao_labels'}})
    {
        my $id = $o_label->GetId();
        if (defined($h_memory_line{$id}))
        {
            $track = $h_memory_line{$id};
        }
        else
        {
            Carp("Unexpected error, the track must be defined for all labels");
        }

        if ($self->{'__mode'} eq 'wrap')    # c'est DisplayFam
        {
            print STDERR "Not implemented yet\n";
            next;
        }
        else
        {
            $o_label->Draw(1, $track, $self->{'__o_gd'});
        }
    }
}

=head3 DrawCoverage

visualisation de la couverture

=cut

sub DrawCoverage
{
    my ($self, %r_opt) = @_;

    my ($limit_left, $limit_top, $limit_right, $limit_bottom) = $self->GetPxLimits();
    my @a_hist = ();

    foreach my $o_segment (@{$self->{'__ao_segments'}})
    {
        for (my $i = $o_segment->{'__begin'}; $i <= $o_segment->{'__end'}; $i++)
        {
            $a_hist[$i]++;
        }
    }
    my $scaling = $self->{'__scaling'};
    my $max;
    for (my $i = 0; $i <= $#a_hist; $i++)
    {
        $max = $a_hist[$i] if ($max > $a_hist[$i]);
    }
    if ($r_opt{'resize'})
    {
        $self->__Resize($self->{'__width'}, $self->{'__margin_top'} + $self->{'__margin_bottom'} + $max + 20);
        ($limit_left, $limit_top, $limit_right, $limit_bottom) = $self->GetPxLimits();
    }
    for (my $i = 0; $i <= $#a_hist; $i++)
    {
        next if ($a_hist[$i] eq '');
        my $x = $limit_left + $i * $scaling;
        $self->{'__o_gd'}->line($x, $limit_bottom, $x, $limit_bottom - $a_hist[$i], $self->{'__h_colors'}{'lightgrey'});
    }
}

sub __Resize
{
    my ($self, $w, $h) = @_;

    delete($self->{'__o_gd'});
    $self->{'__o_gd'} = new GD::Image($w, $h);
    $self->__LoadColorMap();
    $self->__ComputeScaling();
    $self->SetSize($w, $h);
    $self->InitScale();
}

=head3 InitScale

 Title        : InitScale
 Usage        : InitScale($margin_left,$margin_top,$min,$len)
 Prerequisite : none
 Function     : Draw the scale on the display
 Returns      : none
 Args         : $margin_left,	offset from the left (number of pixel)
				$margin_top,	offset from the top (number of pixel)
				$min,			minimum value on the scale
				$len			length of the scale
 Globals      : none

=cut

sub InitScale
{
    my ($self, $margin_left, $margin_top, $min, $len) = @_;

    $margin_left = $self->{'__margin_left'} if (!defined($margin_left));
    $margin_top  = $self->{'__margin_top'}  if (!defined($margin_top));
    my $min_value = 1 if (!defined($min));
    my $max_value = $self->{'__len'} if (!defined($len));

    $self->{'__o_scale'}
      ->Initialize($margin_left, $margin_top, $min_value, $max_value, $self->GetScaling(), $self->{'__o_gd'});
    $self->{'__o_scale'}->Draw($self->{'__h_colors'}{'black'});
}

=head3 __ComputeScaling

 Title        : __ComputeScaling
 Usage        : __ComputeScaling()
 Prerequisite : !!!! Pour le moment ca ne doit marcher que pour Moxy et ClusterMap
 Function     : Compute the scaling factor, based on the left and right margins and on the length to be represented.
 Returns      : none
 Args         : none
 Globals      : none

=cut

sub __ComputeScaling
{
    my ($self) = @_;

    $self->{'__scaling'} =
      ($self->{'__width'} - $self->{'__margin_left'} - $self->{'__margin_right'}) / $self->{'__len'};
}

=head3 Dump

 Title        : Dump
 Usage        : Dump
 Prerequisite : __file_img, __file_fmt, __file_map initialised
 Function     : Save the image and its map into files
 Returns      : none
 Args         : none
 Globals      : none

=cut

sub Dump
{
    my ($self) = @_;

    open(OUT, ">" . $self->{'__file_img'}) or Croak($self->{'__file_img'});
    binmode(OUT);
    if ($self->{'__file_fmt'} eq "gif")
    {
        print OUT $self->{'__o_gd'}->gif;
    }
    elsif ($self->{'__file_fmt'} eq "png")
    {
        print OUT $self->{'__o_gd'}->png;
    }
    close(OUT);

    open(MAP, ">" . $self->{'__file_map'}) or Croak($self->{'__file_map'});
    print MAP $self->{'__map'} . "\n";
    close(MAP);

    print $self->{'__file_img'} . " (" . $self->{'__file_fmt'} . ") created\n" if ($self->{'__verbose'});
}

=head3 SetOutput

 Title        : SetOutput
 Usage        : SetOutput($fimg,$fmap,$fformat)
 Prerequisite : none
 Function     : Initialise the image filename and format and the map filename
 Returns      : none
 Args         : none
 Globals      : none

=cut

sub SetOutput
{
    my ($self, $fimg, $fmap, $fformat) = @_;

    $self->{'__file_img'} = $fimg    if (defined($fimg));
    $self->{'__file_map'} = $fmap    if (defined($fmap));
    $self->{'__file_fmt'} = $fformat if (defined($fformat));    # controler le format
}

=head3 __LoadColorMap

 Title        : __LoadColorMap
 Usage        : __LoadColorMap
 Prerequisite : uses GD functions
 Function     : Load a colormap assotiated to the gd image
 Returns      : none
 Args         : none
 Globals      : none

=cut

sub __LoadColorMap
{
    my ($self) = @_;

    $self->{'__h_colors'}{'white'} = $self->{'__o_gd'}->colorAllocate(@{$self->{'__o_cm'}->{'__h_colors'}{'white'}});
    $self->{'__h_colors'}{'black'} = $self->{'__o_gd'}->colorAllocate(@{$self->{'__o_cm'}->{'__h_colors'}{'black'}});
    foreach my $color (keys(%{$self->{'__o_cm'}->{'__h_colors'}}))
    {
        next if ($color =~ /white|black|lightgrey/);
        $self->{'__h_colors'}{$color} = $self->{'__o_gd'}->colorAllocate(@{$self->{'__o_cm'}->{'__h_colors'}{$color}});
    }
    $self->{'__h_colors'}{'lightgrey'} =
      $self->{'__o_gd'}->colorAllocate(@{$self->{'__o_cm'}->{'__h_colors'}{'lightgrey'}});
    my $transp = $self->{'__h_colors'}{'lightgrey'};
    $self->{'__o_gd'}->setStyle($transp, $transp, gdTransparent, gdTransparent);
}

=head3 SetMargins


=cut

sub SetMargins
{
    my ($self, $l, $t, $r, $b) = @_;

    $self->{'__margin_left'}   = $l if (defined($l));
    $self->{'__margin_top'}    = $t if (defined($t));
    $self->{'__margin_right'}  = $r if (defined($r));
    $self->{'__margin_bottom'} = $b if (defined($b));
    $self->__ComputeScaling();
}

=head3 GetPxLimits
 
=cut

sub GetPxLimits
{
    my ($self) = @_;

    my $top    = $self->{'__margin_top'};
    my $bottom = $self->{'__height'} - $self->{'__margin_bottom'} + 1;
    my $right  = $self->{'__width'} - $self->{'__margin_right'};
    my $left   = $self->{'__margin_left'};
    return ($left, $top, $right, $bottom);
}

=head3 SetScaling
 
=cut

sub SetScaling
{
    my ($self, $s) = @_;

    $self->{'__scaling'} = $s if (defined($s));

}

=head3 GetScaling

=cut

sub GetScaling
{
    my ($self) = @_;

    return $self->{'__scaling'};

}

=head3 SetSize

=cut

sub SetSize
{
    my ($self, $w, $h) = @_;

    $self->{'__width'}  = $w if (defined($w));
    $self->{'__height'} = $h if (defined($h));
}

=head3 SetVerticalStep

=cut

sub SetVerticalStep
{
    my ($self, $s) = @_;

    $self->{'__vstep'} = $s if (defined($s));
}

=head3 GetFiles

=cut

sub GetFiles
{
    my ($self, $mode) = @_;

    if ($mode eq 'basename')
    {
        my ($dir1, $img) = $self->{'__file_img'} =~ /(.+)\/(.+)/;
        $img = $self->{'__file_img'} if ($img eq '');
        my ($dir2, $map) = $self->{'__file_map'} =~ /(.+)\/(.+)/;
        $map = $self->{'__file_map'} if ($map eq '');
        return ($img, $map);
    }
    return ($self->{'__file_img'}, $self->{'__file_map'});
}

=head3 Trace

=cut

sub Trace
{
    my ($self) = @_;

    foreach my $key (sort keys(%$self))
    {
        print STDERR "$key: ", $self->{$key}, "\n";
    }

}

1;
