package iANT::Graphics::Scale;

use Carp;
use GD;
use strict;

sub New
{
    my ($pkg) = @_;

    my $self = {
                __min        => undef,
                __max        => undef,
                __len        => undef,
                __label_low  => 0,
                __label_high => 1,
                __step_low   => undef,
                __step_high  => undef,
                __x          => undef,
                __y          => undef,
                __scaling    => undef,
                __o_gd       => undef,
                __font       => undef
                };
    bless($self, $pkg);

    return ($self);
}

sub Initialize
{
    my ($self, $x, $y, $min, $max, $scaling, $ro_gd) = @_;

    $self->{'__min'} = $min if (defined($min));
    $self->{'__max'} = $max if (defined($max));
    $self->{'__len'} = (abs($self->{'__max'} - $self->{'__min'}) + 1)
      if (defined($self->{'__max'}) && defined($self->{'__min'}));
    $self->{'__label_low'}  = 0;
    $self->{'__label_high'} = 1;
    $self->{'__step_low'}   = 0;
    $self->{'__step_high'}  = 0;
    $self->{'__x'}          = $x if (defined($x));
    $self->{'__y'}          = $y if (defined($y));
    $self->{'__scaling'}    = $scaling if (defined($scaling));
    $self->{'__o_gd'}       = $ro_gd if (defined($ro_gd));
    $self->{'__font'}       = GD::gdSmallFont;

    if (!defined($scaling) && defined($self->{'__o_gd'}))
    {
        my ($width, $height) = $self->{'__o_gd'}->getBounds();
        $self->{'__scaling'} = ($width - $x) / $self->{'__len'};
    }

    $self->SetStepsAuto();
}

sub SetLabels
{
    my ($self, $low, $high) = @_;

    $self->{'__label_low'}  = $low  if (defined($low));
    $self->{'__label_high'} = $high if (defined($high));
}

sub SetSteps
{
    my ($self, $low, $high) = @_;

    $self->{'__step_low'}  = $low  if (defined($low));
    $self->{'__step_high'} = $high if (defined($high));
}

sub SetStepsAuto
{
    my ($self) = @_;

    return if (!defined($self->{'__len'}));

    #return if ( defined($self->{'__step_low'}) && defined($self->{'__step_high'}) );
    my ($high) = length($self->{'__len'}) - 2;    # oui c'est ca on fait le calcul sur la longueur des labels
    my ($low)  = length($self->{'__len'}) - 3;

    if ($high < 2)
    {
        $high = 2;
        $low  = 1;
    }
    $self->SetSteps(10**$low, 10**$high);
}

sub Draw
{
    my ($self, $color) = @_;

    return if (!$self->__CheckConsistency("warning"));

    my $rogd    = $self->{'__o_gd'};
    my $x       = $self->{'__x'};
    my $y       = $self->{'__y'};
    my $len     = $self->{'__len'};
    my $scaling = $self->{'__scaling'};

    my %scale;
    $scale{'low'} = {
                     'step'   => int($self->{'__step_low'}),
                     'label'  => $self->{'__label_low'},
                     'height' => 5
                     };
    $scale{'high'} = {
                      'step'   => int($self->{'__step_high'}),
                      'label'  => $self->{'__label_high'},
                      'height' => 10
                      };

    foreach my $lev ('low', 'high')
    {
        for (my $i = 1; $i <= ($len / $scale{$lev}{step}); $i++)
        {
            my $step     = $scale{$lev}{step};
            my $rescaled = $scaling * $i * $step;

            # 			print STDERR "$i\nlabel/$lev ",$scale{$lev}{label},"\n";
            # 			print STDERR "step/$lev $step; rescaled=$rescaled\n";
            if ($scale{$lev}{label})
            {
                my $label = $self->{'__min'} + $i * $step - 1;
                $rogd->string($self->{'__font'}, $x + $rescaled - 5, $y - 25, $label, $color);
            }
            $rogd->line($x + $rescaled, $y, $x + $rescaled, $y - $scale{$lev}{'height'}, $color);

            #pas vraiment une echelle $rogd->line($x+&jScaling($i*$sh),$y,$x+$rescaled,$y+$haut,gdStyled);
        }
    }
    $rogd->line($x, $y, $x + $scaling * $len, $y, $color);
}

sub Trace
{
    my ($self) = @_;

    foreach my $key (sort keys(%$self))
    {
        print "$key: ", $self->{$key}, "\n";
    }

}

sub __CheckConsistency
{
    my ($self, $action) = @_;

    foreach my $key (keys(%{$self}))
    {
        if (!defined($self->{$key}))
        {
            if ($action eq "die")
            {
                Croak("Fatal error: consistency check failed => >$key< not defined\n");
            }
            else
            {
                print STDERR "Warning: consistency check failed => >$key< not defined, unable to draw the scale\n";
                return 0;
            }
        }
    }
    return 1;
}

1;
