package iANT::HyperlinkDB;

require 'UNIVERSAL.pl';
use strict;
use warnings;
use Carp;
use XML::Simple;

=head1

	<hyperlink id="InterPro">
		<priority>1</priority>
		<target>win_db</target>
		<regexp_acc>(IPR\d{5})</regexp_acc>
		<regexp_id/>
		<url>http://srs6.ebi.ac.uk/srs6bin/cgi-bin/wgetz?-id+4Flds1EnCA0+[interpro-AccNumber:~MASKACC~]</url>
		<label>~MASKACC~</label>
	</hyperlink>

	& => &amp;

=cut

sub _Init
{
    my ($self, %h_param) = @_;

    $self->{'__xs'}                = undef;
    $self->{'__r_xml_hyperlinkdb'} = undef;
    if (!defined($h_param{'db'}))
    {
        croak("Fatal Error: the parameter 'db' must be specified\n");
    }
    $self->__Load($h_param{'db'});
}

sub __Load
{
    my ($self, $filename) = @_;

    $self->{'__xs'} = new XML::Simple(SuppressEmpty => '');   #  KeyAttr => 'hyperlink' ); #, ForceArray => [ 'Hit' ] );
    $self->{'__r_xml_hyperlinkdb'} = $self->{'__xs'}->XMLin($filename)->{hyperlink};
}

sub Add
{
    my ($self, $db, %h_param) = @_;

    foreach my $key (keys(%h_param))
    {
        $self->{'__r_xml_hyperlinkdb'}->{$db}->{$key} = $h_param{$key};
    }
}

sub GetHyperlink
{
    my ($self, $txt, $db) = @_;
    my $html = $txt;

    if ($db eq 'auto')
    {
    }
    else
    {
        return $txt if (!defined($self->{'__r_xml_hyperlinkdb'}->{$db}));
        my $regexp_acc = $self->{'__r_xml_hyperlinkdb'}->{$db}->{'regexp_acc'};
        my $regexp_id  = $self->{'__r_xml_hyperlinkdb'}->{$db}->{'regexp_id'};
        my $priority   = $self->{'__r_xml_hyperlinkdb'}->{$db}->{'priority'};
        my $url        = $self->{'__r_xml_hyperlinkdb'}->{$db}->{'url'};
        my $label      = $self->{'__r_xml_hyperlinkdb'}->{$db}->{'label'};
        my $win_target = $self->{'__r_xml_hyperlinkdb'}->{$db}->{'target'};
        my $newlabel   = $label;
        my $newurl     = $url;
        while ($regexp_acc ne "" && $txt =~ /$regexp_acc/g)
        {
            my $acc = $1;
            my $subs = $self->__BuildUrl($acc, "~MASKACC~", $newurl, $newlabel, $win_target);
            $html =~ s,$acc,$subs,g;
        }
        while (defined($regexp_id) && $regexp_id ne "" && $txt =~ /$regexp_id/g)
        {
            my $id = $1;
            my $subs = $self->__BuildUrl($id, "~MASKID~", $newlabel, $newurl);
            $html =~ s,$id,$subs,g;
        }

        #print STDERR "$regexp_acc\n$regexp_id\n$priority\n$url\n$label\n";
        print STDERR "$html\n";
    }
}

sub __BuildUrl
{
    my ($self, $tag, $mask, $url, $label, $win_target) = @_;

    if ($label ne "")
    {
        $label =~ s/$mask/$tag/;
    }
    else
    {
        $label = $tag;
    }
    $url =~ s/$mask/$tag/;
    my $newtarget = (defined($win_target) && $win_target ne '') ? " target=$win_target " : "";
    return "<a href=\"$url\"$newtarget>$label</a>";
}

1;
