#!/usr/bin/env perl

package iANT::Project;

use strict;
use warnings;
use Carp;
use XML::Twig;
use iANT::Common;
use iANT::Project::User;
use iANT::project::project;
use ParamParser;

require 'UNIVERSAL.pl';


BEGIN
{
    our $VERSION = "0.1";
}

=head1 NAME

iANT::Project - iANT/MENS Project object


=head1 SYNOPSIS


example: 


=cut




=head1 PUBLIC METHODS

=pod 

=head3 function New

 Title        : New
 Usage        : New iANT::Project;
 Prerequisite : none
 Function     : Create an object iANT::Project
 Returns      : An object iANT::Project
 Args         : none
 Globals      : none

=cut

sub _Init
{
    my ($self, $source) = (@_);

    $self->{__title} = "";
    $self->{__data_dir} = "";
    $self->{__cfg_file} = "";
    $self->{__directory} = "";
    $self->{__ra_users} = [];
    
    
    if (defined $source)
    {
        $self = &Update($self, $source);
    }

}

=pod

=head3 function Update

 Title        : Update
 Usage        : $o_project->Update($source);
 Prerequisite : none
 Function     : Update attributes of an existing Project object
 Returns      : An object Project
 Args         : $source: config file | ParamParser object | hashref
 Globals      : none

=cut

sub Update
{
    my ($self, $source) = (@_);
    my $o_xml_entry;
    
    my $rh_param = {};
  
    if (ref($source) =~ /hash/i)
    {
        $self = &__InitializeFromHash($self, $source);
    }
    elsif (ref($source) =~ /paramparser/i)
    {
        #traitement des auth_level_\d+

        $source->Dump('HASH', $rh_param,'');
        $rh_param->{'users'} = &__Users($source);
        $self = &__InitializeFromHash($self, $rh_param);
    }
    elsif (($source !~ /\n/) && (-e $source))
    {
        my $o_param = New ParamParser($source);
        #traitement des auth_level_\d+
        

        $o_param->Dump('HASH', $rh_param,'');
        $rh_param->{'users'} = &__Users($o_param);
        
        $self = &__InitializeFromHash($self, $rh_param);
        $self->{__cfg_file} = $source;
    }
    else
    {
        confess("Unknown source");
    }
    return $self;
}

sub __Users
{
    my $o_param = shift;
    my $rh_users = {};

    $o_param->SelectNameSpace("auth_level_");
    foreach my $level ($o_param->GetKeys())
    {
        my $users_string = $o_param->Get($level);
        $users_string =~ s/\s//g;
        my @a_users = split (/,/,$users_string);
        foreach my $user (@a_users)
        {
            $rh_users->{$user} = {level => $level};
        }
    }
    $o_param->SelectNameSpace("");
    return $rh_users;

}

=pod

=head3 function GetXml

 Title        : GetXml
 Usage        : $o_project->GetXml($target_element);
 Prerequisite : none
 Function     : Retrieve an Project (or part of) in xml format
 Returns      : An xml string
 Args         : $target_element: optional; if empty or undef, retrieve all
                xml elements.
 Globals      : none

=cut

sub GetXml
{
    my ($self, $target_element) = @_;
    my $o_xml_project = iANT::project::project->new();

    if (   (!defined $target_element)
        || ($target_element eq 'project')
        || ($target_element eq '')
        || ($target_element eq 'all'))
    {
        $o_xml_project->title($self->{__title});
        $o_xml_project->data_dir($self->{__data_dir});
        $o_xml_project->auth_edit($self->{__cfg_file});
        $o_xml_project->auth_edit($self->{__directory});
        $o_xml_project->auth_view(join(",",@{$self->{__ra_auth_view}}));
        $o_xml_project->auth_edit(join(",",@{$self->{__ra_auth_edit}}));
        
    }

    if (   (!defined $target_element)
        || ($target_element eq 'user')
        || ($target_element eq '')
        || ($target_element eq 'all'))
    {
        if (defined $self->{__ra_users})
        {
            my @a_o_user;
            my @a_ro_user = @{$self->{__ra_users}};
            foreach my $o_user (@a_ro_user)
            {
                my $o_xml_user = $o_user->GetXml();
                push(@a_o_user, $o_xml_user);
            }
            $o_xml_project->link(@a_o_user);
        }
    }


    return $o_xml_project;
}

=pod

=head3 procedure Dump

 Title        : Dump
 Usage        : $o_entry->Dump();
 Prerequisite : uses iANT::Common::Dump method
 Function     : Dump an Project in xml format
 Returns      : none
 Args         : $target:-optional; if empty or undef, print on STDOUT
                        -if defined, print in the specified file (unix-like mode is mandatory)
 Globals      : none

=cut

sub Dump
{
    my ($self, $target) = (@_);
    return &iANT::Common::Dump(@_);
}

=pod
 
=head3 Getters and Setters

=item Get

    Function     : Generic Getter
    Args         : $attribute
    Prerequisite : uses iANT::Common::Get method

=cut

sub Get
{
    my ($self, $attribut) = @_;
    return &iANT::Common::Get(@_);
}

=pod

=item Set

    Function    : Generic Setter
    Args        : $attribute
                  $value
    Prerequisite : uses iANT::Common::Set method

=cut

sub Set
{
    my ($self, $attribut, $value) = @_;
    &iANT::Common::Set(@_);

}


sub Restore
{
    my ($self) = @_;
    return &iANT::Common::Restore(@_);
    
}


=item GetTitle

    Returns      : Project title

=cut

sub GetTitle
{
    my ($self) = @_;
    return $self->Get('title');
}

=item SetTitle

    Set Project title

=cut

sub SetTitle
{
    my ($self, $title) = @_;
    $self->Set('title', $title);
}

=item GetDataDir

    Returns      : Project Data directory

=cut

sub GetDataDir
{
    my ($self) = @_;
    return $self->Get('data_dir');
}

=item SetDataDir

    Set Project Data directory

=cut

sub SetDataDir
{
    my ($self, $data_dir) = @_;
    $self->Set('data_dir', $data_dir);
}


=item GetDirectory

    Returns      : Project directory

=cut

sub GetDirectory
{
    my ($self) = @_;
    return $self->Get('directory');
}

=item SetDirectory

    Set Project directory

=cut

sub SetDirectory
{
    my ($self, $directory) = @_;
    $self->Set('directory', $directory);
}


=item GetCfgFile

    Returns      : Project Config file

=cut

sub GetCfgFile
{
    my ($self) = @_;
    return $self->Get('cfg_file');
}

=item SetCfgFile

    Set Project Config file

=cut

sub SetCfgFile
{
    my ($self, $cfg_file) = @_;
    $self->Set('cfg_file', $cfg_file);
}


=item GetUsers

    Returns      : Project Users objects

=cut

sub GetUsers
{
    my ($self) = @_;
    return $self->Get('ra_users');
}

=item SetUsers

    Add a Project User object

=cut

sub SetUser
{
    my ($self, $o_user) = @_;
    my @a_o_users = ();
    if (defined $self->{__ra_users} && ($self->{__ra_users} ne ''))
    {
        @a_o_users = @{$self->{__ra_users}};
    }
    push(@a_o_users, $o_user);
    $self->{__ra_users} = \@a_o_users;
    return;

}

=item GetAuthView

    Returns      : Project Authorized Viewers

=cut

sub GetUser
{
    my ($self, $login) = @_;
    
    if (! defined $login || ($login eq ''))
    {
        confess('GetUser needs a login ');
    }
    my $ra_o_users = $self->GetUsers();
    my $level = 0;
    if (defined $ra_o_users && ($ra_o_users ne '') && (ref($ra_o_users) =~ /array/i))
    {
        foreach my $o_user (@{$ra_o_users})
        {
            my $match_selection_parameters = iANT::Common::IsMatchingSelectionParameter($o_user, {login => $login});

            if ($match_selection_parameters)
            {
                return $o_user;
                
            }
        }
    }
    return;
}


sub GetUserLevel
{
    my ($self, $login) = @_;
    
    if (! defined $login || ($login eq ''))
    {
        confess('GetUserLevel needs a login ');
    }

    my $o_user = $self->GetUser($login);
    my $level;
    if (defined $o_user)
    {
        $level = $o_user->GetLevel();
    }
    $level = 0 if (! defined $level);
    return $level;
}

=head1 PRIVATE METHODS

=pod

=head3 function __InitializeFromHash

 Title        : __InitializeFromHash
 Usage        : __InitializeFromHash($o_project,$rh_attributes);
 Prerequisite : none
 Function     : Fill attributes of an existing Project object from a hash (attr => value, ...)
 Returns      : An object Project
 Args         : $o_project:
                $rh_attributes
 Globals      : none

=cut

sub __InitializeFromHash
{
    my ($self, $rh_attributes) = (@_);

    $self->{__title}     = $rh_attributes->{title}     if (defined $rh_attributes->{title});
    $self->{__data_dir}  = $rh_attributes->{data_dir}  if (defined $rh_attributes->{data_dir} && -d $rh_attributes->{data_dir});
    $self->{__directory}  = $rh_attributes->{directory}  if (defined $rh_attributes->{directory} && -d $rh_attributes->{directory});
    $self->{__cfg_file}  = $rh_attributes->{cfg_file}  if (defined $rh_attributes->{cfg_file} && -e $rh_attributes->{cfg_file});
    $self->{__ra_users} = [];
    $self->{__ra_users} = $rh_attributes->{ra_users}
      if ((defined $rh_attributes->{ra_users}) && (ref($rh_attributes->{ra_users}) =~ /array/i));
    
    if (defined $rh_attributes->{users} && (ref ($rh_attributes->{users}) =~ /hash/i))
    {
        foreach my $login (keys %{$rh_attributes->{users}})
        {
            my $o_user = New iANT::Project::User({login => $login, level => $rh_attributes->{users}->{$login}->{level}});
            push (@{$self->{__ra_users}}, $o_user);
        }
    }

    return $self;
}


=head1 AUTHOR

    Sebastien Carrere (Sebastien.Carrere@toulouse.inra.fr)
    Jerome Gouzy (Jerome.Gouzy@toulouse.inra.fr)

=head1 MANDATORY PERL PACKAGES

    Class::XML
    XML::Twig
    Carp
    ParamParser


=head1 SEE ALSO


=cut

1;
