/*
   $Id: ParametersConvert.cpp 648 2011-10-31 12:59:22Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#include "ParametersConvert.h"
#include "CommandConvert.h"

CSimpleOpt ParametersInitializerConvert::_Init(int argc, char* argv[], const Command& cmd)
{
	CSimpleOpt::SOption options[] = {
	    { OPT_OUTPUTFORMAT,  (char*)"-m",    SO_REQ_SEP }, // "-m"
	    { OPT_OUTPUTFORMAT,  (char*)"--output-format",SO_REQ_SEP }, // "--output-format"
	    { OPT_CHUNKSIZE,     (char*)"--chunk-size",SO_REQ_SEP    }, // "--output-format"
	    { OPT_INPUTFORMAT,   (char*)"--input-format", SO_REQ_SEP }, // "--input-format"
	    { OPT_RESULTSFILENAME,  (char*)"-o", SO_REQ_SEP }, // "-o"
	    { OPT_RESULTSFILENAME,  (char*)"--output-file", SO_REQ_SEP }, // "--output-file"
	    { OPT_SILENT,  (char*)"--silent",   SO_NONE }, //
	    { OPT_VERBOSE,  (char*)"--verbose", SO_NONE }, //
	    { OPT_VERBOSE,  (char*)"-v",        SO_NONE }, //
	    { OPT_PAIREND, (char*)"--pair-end", SO_NONE  }, // "--pair-end"
		{ OPT_COMPRESS,   (char*) "--compress",     SO_REQ_SEP },
	    { OPT_HELP, (char*)"-?",     SO_NONE    }, // "-?"
	    { OPT_HELP, (char*)"-h",     SO_NONE    }, // "-?"
	    { OPT_HELP, (char*)"--help", SO_NONE    }, // "--help"
	    
	    SO_END_OF_OPTIONS                       // END
	};

	//p.__Help = help;
	p.__argc = argc;

	// declare our options parser, pass in the arguments from main
    // as well as our array of valid options.
    CSimpleOpt args(argc, argv, options);
	while (args.Next()) {
		if (args.LastError() == SO_SUCCESS) 
		{
			if (args.OptionId() == OPT_HELP) 
			{
				cmd.Usage();
				exit(0);
			}
			else if (args.OptionId() ==  OPT_VERBOSE )
			{
				p.__verbose = true;
			}
			else if (args.OptionId() ==  OPT_SILENT )
			{
				p.__silent = true;
			}
			else if ( args.OptionId() == OPT_COMPRESS )
			{
				p.__results_filename.SetCompression(args.OptionArg());
				p.__results_bwd_filename.SetCompression(args.OptionArg());
			}
			else if (args.OptionId() ==  OPT_PAIREND )
			{
				p.__pair_end = true;
			}
            else if (args.OptionId() == OPT_OUTPUTFORMAT)
            {
				string out = args.OptionArg();
				if ( ToLower(out) == "fa" || ToLower(out) == "fasta" )
				{
					p.__output_format = OUTPUT_FORMAT_FA;
				}
				else if ( ToLower(out) == "fastq" )
				{
					p.__output_format = OUTPUT_FORMAT_FASTQ;
				}
				else if ( ToLower(out) == "sam" )
				{
					p.__output_format = OUTPUT_FORMAT_SAM;
				}
				else if ( ToLower(out) == "bam" )
				{
					p.__output_format = OUTPUT_FORMAT_BAM;
				}
				else if ( ToLower(out) == "samext" )
				{
					p.__output_format = OUTPUT_FORMAT_SAMEXT;
				}
				else if ( ToLower(out) == "bamext" )
				{
					p.__output_format = OUTPUT_FORMAT_BAMEXT;
				}
				else if ( ToLower(out) == "bed" )
				{
					p.__output_format = OUTPUT_FORMAT_BED;
				}
				else if ( out == "0" )
				{
					p.__output_format = OUTPUT_FORMAT_M0;
				}
				else
				{
					int output_format = atoi(args.OptionArg());
					if (output_format==0)
					{
						p.__output_format = OUTPUT_FORMAT_ILLEGAL;
					}
					else
					{
						p.__output_format = output_format;
					}
				}
            }
            else if (args.OptionId() == OPT_CHUNKSIZE)
            {
				p.__output_chunk_size = atoi(args.OptionArg());
			}
            else if (args.OptionId() == OPT_INPUTFORMAT)
            {
				string in = args.OptionArg();
				if ( ToLower(in) == "fa" || ToLower(in) == "fasta")
				{
					p.__input_format = INPUT_FORMAT_FA;
				}
				else if ( ToLower(in) == "fastq" )
				{
					p.__input_format = INPUT_FORMAT_FASTQ;
				}
				else if ( ToLower(in) == "sam" || ToLower(in) == "samext")
				{
					p.__input_format = INPUT_FORMAT_SAM;
				}
				else if ( ToLower(in) == "bam" || ToLower(in) == "bamext" )
				{
					p.__input_format = INPUT_FORMAT_BAM;
				}
				else
				{
					string msg = "ERROR - ";
					msg += in;
					msg += " Invalid value for --input-format";
					throw(runtime_error(msg));
				}
            }          
            else if (args.OptionId() == OPT_RESULTSFILENAME )
            {
				string results_filename = (string) args.OptionArg();
				string bwd_filename;
				p.__ParseFileName(results_filename,bwd_filename);
				p.__results_filename     = (CompressedFileName) results_filename;
				p.__results_bwd_filename = (CompressedFileName) bwd_filename;
			}

			#ifdef DEBUGLOG
			else if (args.OptionId() ==  OPT_LOG_LEVEL)
			{
				p.__log_level = (string) args.OptionArg();
			}
			#endif
		}
        else 
        {
            fprintf(stderr,"Invalid argument: %s\n", args.OptionText());
            fprintf(stderr,"Please use option --help to see the usage\n");
            exit(1);	
        }
    }
    return args;
}

/**
 * \brief called by Init to finish the job
 */
void ParametersInitializerConvert::_Init2 ( const CSimpleOpt& args, const Command& cmd)
{
	if (args.FileCount()==0)
	{
		p.__query_filename = (CompressedFileName) "-";
		p.__bank_filename  = (CompressedFileName) "-";
	}
	else if ( args.FileCount()==1)
	{
		string query_filename = args.File(0);
		string bwd_filename   = "";
		p.__ParseFileName(query_filename,bwd_filename);
		p.__query_filename = (CompressedFileName) query_filename;
		p.__bwd_filename   = (CompressedFileName) bwd_filename;
	}
	else
	{		
		 fprintf(stderr,"ERROR - Zero of one filename is required \n\n");
		 cmd.Usage();
		 exit(1);
	}
	
	if ( p.__input_format==INPUT_FORMAT_UNKNOWN )
	{
		p.__input_format = DEFAULT_GLINTCONVERT_INPUT_FORMAT;
	}
	if ( p.__output_format==OUTPUT_FORMAT_UNKNOWN )
	{
		p.__output_format = DEFAULT_GLINTCONVERT_OUTPUT_FORMAT;
	}
}

/** 
 *  \brief Validate the parameters useful for glintindex
 *  \return true if all good
*/
bool ParametersInitializerConvert::_Validation()
{
	bool ok = true;	

	if ( p.__bwd_filename )
	{
		cerr << "ERROR - You should NOT have two input files with glintconvert\n";
		ok = false;
	}
	if ( (p.__input_format == INPUT_FORMAT_FA || p.__input_format == INPUT_FORMAT_FASTQ) && p.__pair_end )
	{
		cerr << "ERROR - The mode pair-end may be used ONLY with bam/sam input\n";
		ok = false;
	}
	if ( (p.__input_format == INPUT_FORMAT_FA || p.__input_format == INPUT_FORMAT_FASTQ) && 
	    !(p.__output_format == OUTPUT_FORMAT_FA || p.__output_format == OUTPUT_FORMAT_FASTQ) )
    {
		cerr << "ERROR - ONLY fasta/fastq output formats may be used with fasta/fastq input formats\n";
	}
	if ( p.__results_bwd_filename && !p.__pair_end )
	{
		cerr << "ERROR - You should NOT have two output files (" << p.__results_filename.GetName();
		cerr << " and " << p.__results_bwd_filename.GetName() << ") if not in pair-end mode\n\n";
		ok = false;
	}
	if ( !p.__results_bwd_filename && p.__pair_end && (p.__output_format == OUTPUT_FORMAT_FA || p.__output_format == OUTPUT_FORMAT_FASTQ))
	{
		cerr << "ERROR - You SHOULD HAVE have two output files in output format fasta or fastq in pair-end mode\n";
		ok = false;
	}

	return ok;
}
