/*
   $Id: ParametersFreq.cpp 648 2011-10-31 12:59:22Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/
#include "ParametersFreq.h"
#include "Command.h"

CSimpleOpt ParametersInitializerFreq::_Init(int argc, char* argv[], const Command& cmd)
{
	CSimpleOpt::SOption options[] = {
	    // ID       TEXT          TYPE
	    { OPT_COMPLEXITY,    (char*) "-C",   SO_REQ_SEP }, // "-C"
	    { OPT_LC_FILTERING,  (char*) "--no-lc-filtering", SO_NONE }, // "--no-lc-filtering"
	    { OPT_SILENT,  (char*)"--silent",   SO_NONE }, //
	    { OPT_VERBOSE,  (char*)"--verbose", SO_NONE }, //
	    { OPT_VERBOSE,  (char*)"-v",        SO_NONE }, //
	
	#ifdef DEBUGLOG
	    { OPT_LOG_LEVEL,  (char*)"-D",     SO_REQ_SEP }, // "-D"
	#endif
	
		// writing compressed data
		{ OPT_COMPRESS,   (char*) "--compress",     SO_REQ_SEP },
		
	    // To get some help
	    { OPT_HELP, (char*)"-?",     SO_NONE    }, // "-?"
	    { OPT_HELP, (char*)"-h",     SO_NONE    }, // "-?"
	    { OPT_HELP, (char*)"--help", SO_NONE    }, // "--help"

    	{ OPT_NOGAP,  (char*)"--no-gap",    SO_NONE }, //

	    { OPT_STRICT,  (char*)"--strict",   SO_NONE     }, //

	    { OPT_TOUPPER, (char*) "--to-upper", SO_NONE }, // "--to-upper"

		{ OPT_COUNT_CUTOFF, (char*)"--count_cutoff", SO_REQ_SEP    }, 
		{ OPT_CONDEMNED_MIN_SIZE, (char*)"--condemned_min_size", SO_REQ_SEP    }, 
		{ OPT_CONSEQFREQ, (char*)"--conseqFreq", SO_REQ_SEP    }, 
		{ OPT_CONSEQUNFREQ, (char*)"--conseqUnFreq", SO_REQ_SEP    }, 
		{ OPT_SUITABLE_MIN_SIZE, (char*)"--suitable_min_size", SO_REQ_SEP    }, 
	    
	    SO_END_OF_OPTIONS                       // END
	};

	//p.__Help = help;
	p.__argc = argc;

	// declare our options parser, pass in the arguments from main
    // as well as our array of valid options.
    CSimpleOpt args(argc, argv, options);
	while (args.Next()) {
		if (args.LastError() == SO_SUCCESS) 
		{
			if (args.OptionId() == OPT_HELP) 
			{
				cmd.Usage();
				exit(0);
			}
			else if (args.OptionId() == OPT_COMPLEXITY )
			{
				p.__complexity = atoi(args.OptionArg());
			}
			else if (args.OptionId() == OPT_LC_FILTERING )
			{
				p.__complexity = COMPLEXITY_NONE;
			}
			else if (args.OptionId() ==  OPT_VERBOSE )
			{
				p.__verbose = true;
			}
			else if (args.OptionId() ==  OPT_SILENT )
			{
				p.__silent = true;
			}
            else if (args.OptionId() ==  OPT_NOGAP )
            {
            	p.__dyn = false;
            }
            else if (args.OptionId() ==  OPT_TOUPPER )
            {
            	p.__to_upper = true;
            }
            else if (args.OptionId() ==  OPT_STRICT )
            {
            	p.__strict = true;
            }
			else if ( args.OptionId() == OPT_COMPRESS )
			{
				p.__results_filename.SetCompression((string) args.OptionArg());
			}
			else if (args.OptionId() ==  OPT_COUNT_CUTOFF )
            {
				p.__count_cutoff = atoi(args.OptionArg());
			}
			else if (args.OptionId() ==  OPT_CONDEMNED_MIN_SIZE )
            {
				p.__condemned_min_size = atoi(args.OptionArg());
			}
			else if (args.OptionId() ==  OPT_CONSEQFREQ )
            {
				p.__conseq_freq = atoi(args.OptionArg());
			}
			else if (args.OptionId() ==  OPT_CONSEQUNFREQ )
            {
				p.__conseq_un_freq = atoi(args.OptionArg());
			}
			else if (args.OptionId() ==  OPT_SUITABLE_MIN_SIZE )
            {
				p.__suitable_min_size = atoi(args.OptionArg());
			}

			#ifdef DEBUGLOG
			else if (args.OptionId() ==  OPT_LOG_LEVEL)
			{
				p.__log_level = (string) args.OptionArg();
			}
			#endif
			
		}
        else 
        {
            fprintf(stderr,"Invalid argument: %s\n", args.OptionText());
            fprintf(stderr,"Please use option --help to see the usage\n");
            exit(1);	
        }
    }
    return args;
}

/**
 * \brief called by _Init to finish the job
 */
void ParametersInitializerFreq::_Init2 ( const CSimpleOpt& args, const Command& cmd )
{
	if ( args.FileCount()>=1 )
	{
		p.__bank_filename = (CompressedFileName) args.File(0);
	}

	if ( args.FileCount() >= 2 )
	{		
	    p.__query_filename = (CompressedFileName) args.File(1);
	}

     if ( !p.__pipe )
     {
		 if ( p.__results_filename.GetCompression() == NOCOMPRESSION )
		 {
			 p.__results_filename.SetCompression( p.__query_filename.GetCompression() );
		 }
	 }


	p.__input_format  = INPUT_FORMAT_FA;
	p.__output_format = OUTPUT_FORMAT_FA;
	p.__index_mode    = false;
}

/** 
 *  \brief Validate the parameters useful for glintindex
 *  \return true if all good
*/
bool ParametersInitializerFreq::_Validation()
{
	bool ok = true;
	if ( p.__argc == 1 )
	{
		ok = false;
	}
    if (!p.__bank_filename)
    {
		cerr << "ERROR - Illegal bank file name\n\n";
		ok = false;
	};
	return ok;
}

