/* 
  $Id: QueryBufferBank.cpp 648 2011-10-31 12:59:22Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#include <vector>
#include "QueryBuffer.h"
#include "QueryBufferBank.h"
#include "CompactTree.h"
#include "Bank.h"
using namespace std;

extern RealLogger* logger;


/**
 * \brief QueryBufferBank constructor
 * \param The name of the bank files (several files with different extension added)
 */

QueryBufferBank::QueryBufferBank(const string& bank_filename, CompressionAlgorithm compression):
	seq_info(bank_filename, compression),
	seq_cnt(0),
	seq_nb(seq_info.size()),
	__fd_in(_file_descriptors.back().fd)
{
	__fd_in = FOpen( bank_filename, FA_EXTENSION,"r", compression );
}

/**
 * \brief Read the next sequence from the .fa bank file, and copy the sequence to the buffer
 * \return true if a sequence was read, false if not (End of file)
 */

bool QueryBufferBank::ReadNextSeq()
{
	signed char c;
	
	_Clear();

	// The FIRST character should be a '$'
	c=fgetc(__fd_in);
	
	if ( c==EOF )
	{
		return false;
	}
	
	if ( c != '$')
	{
		string msg = "ERROR - Bad format file";
		throw ios_base::failure(msg.c_str());
	}

	// skip the separators
	while ( (c=fgetc(__fd_in)) != EOF && c == '$' ) {};
	ungetc(c,__fd_in);
	
	// Read the sequence
	while ( (c=fgetc(__fd_in)) != EOF && c != '$' )
	{
		_Putc(c);
	}
	if ( c==EOF )
	{
//		FClose(__fd_in);
		return false;
	}
	else
	{
		ungetc(c,__fd_in);
		
		// There is a new sequence in the buffer: put a separator, set the sequence parameters and return true
		_PutSeparator();
		_SetSeqId(seq_info[seq_cnt++].Name);
		_SetSeqLength(_query_bfr.size() - 2*sizeof(unsigned char)/sizeof(unsigned char)); // Taking into account the separators
		_SetBase((unsigned char*)_query_bfr.GetBase());
		return true;
	}
}

/**
 * \brief Constructor. There is no parameter to pass, because the data are taken from the Bank singleton.
 */
QueryBufferSelf::QueryBufferSelf(): 	
	__ctree(CompactTree::Instance())
{	
	_SetBase((unsigned char*)__ctree.GetSeq());
	_SetSeqLength(__ctree.GetSequenceLength());
}

/**
 * \brief The whole genome sequence is already in memory: we take this sequence as the only sequence
 * \return true for the first call, false otherwise
 */
bool QueryBufferSelf::ReadNextSeq()
{
	static bool done = false;
	bool rvl = !done;
	done = true;
	return rvl;
}
