/*
   $Id: memory.cpp 649 2011-10-31 13:57:42Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#include <cerrno>
#include <unistd.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/mman.h>
#include <limits>
using namespace std;

#include "memory.h"
#include "utils.h"

void* MemoryPool::Malloc(size_t sze)
{
    if (pool_top[current_pool]+sze >= pool[current_pool] + pool_size)        // We must change our pool to alloc more data
    {
        current_pool++;
        //void* base = memory_survey(pool_size,"");
        void* base = MAlloc(pool_size);
        mem_alloc += pool_size;
        pool.push_back((uint8_t*)base);
        pool_top.push_back((uint8_t*)base);
    };
    uint8_t* rvl = pool_top[current_pool];
    pool_top[current_pool] += sze;
    mem_used += sze;
    return (void*)rvl;
}

void MemoryPool::FreeAll()
{
    while (current_pool>0) {
        free(pool[current_pool--]);
        pool.pop_back();
        pool_top.pop_back();
    }
    pool_top[current_pool]=pool[current_pool];
    mem_used = 0;
    mem_alloc = pool_size;
}

#ifdef REMOVED
size_t MaxSize(int part=0)
{
    size_t max_size;
    numeric_limits<ssize_t> limit_ssize_t;

    const int part_limit = 80;            /* default when using limit to discover the available memory */
    const int part_phys_mem=50;           /* default when using sysconf to discover the available memory */

    struct rlimit memoryuse,datasize;

    int rvl = getrlimit(RLIMIT_DATA,&datasize);
    if (rvl != 0)
    {
		string msg = (string)"ERROR - " + strerror(errno) + " ";
		throw(run_time(msg));
	}

    rvl = getrlimit(RLIMIT_RSS,&memoryuse);
    if (rvl != 0)
    {
		string msg = (string)"ERROR - " + strerror(errno) + " ";
		throw(run_time(msg));
	}

    /* No limit specified: try to know the total physical memory */
    if (datasize.rlim_cur==RLIM_INFINITY && memoryuse.rlim_cur==RLIM_INFINITY)
    {
        if (part==0)
        {
            part = part_phys_mem;
        };
        long page_size   = sysconf(_SC_PAGESIZE);
        long nb_of_pages = sysconf(_SC_PHYS_PAGES);
        long physical_memory = nb_of_pages / 100;
        physical_memory *= page_size;
        if (physical_memory < limit_ssize_t.max()/part)
        {
            physical_memory *= part;
        }
        else
        {
            physical_memory = limit_ssize_t.max();
        }
        max_size = physical_memory;
    }
    else
    {
        if (part==0)
        {
            part = part_limit;
        };
    };

    if (datasize.rlim_cur==RLIM_INFINITY && memoryuse.rlim_cur!=RLIM_INFINITY)  /* Limits for only 1 parameter */
    {
        max_size = memoryuse.rlim_cur/100;

        max_size *= part;
    };
    if (datasize.rlim_cur!=RLIM_INFINITY && memoryuse.rlim_cur==RLIM_INFINITY)
    {
        max_size = datasize.rlim_cur/100;
        max_size *= part;
    };
    if (datasize.rlim_cur!=RLIM_INFINITY && memoryuse.rlim_cur!=RLIM_INFINITY)  /* Limits for both parameters: take the lowest */
    {
        max_size = (datasize.rlim_cur < memoryuse.rlim_cur) ? datasize.rlim_cur/100 : memoryuse.rlim_cur/100;
        max_size *= part;
    };

    /* Now, we ask the system to give us those pages, then we free them */

    void * base = malloc(max_size);
    if (base==0) rvl=-1;
    if (rvl != 0)
    {
		string msg = (string)"ERROR - " + strerror(errno) + " ";
		throw(run_time(msg));
	}

    free(base);
    return max_size;
};

#endif
