/*
   $Id: rtstack.h 648 2011-10-31 12:59:22Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#ifndef __RTSTACK_HPP__
#define __RTSTACK_HPP__ 1

/*
    rtstack derives from runtime_error, but the constructor stracks the call stack
    it will then be printed by the function what()
    
    NOTES 1/ This code works ONLY with Gcc 
          2/ You must link with the -rdynamic switch
          
    USE:
    
        #include "rtstack.hpp"

        try
        {
             ...your code...
             string msg = "ERROR - bla bla\n";
             msg += ERROR_FILE_LINE(__FILE__,__LINE__);
             throw (rtstack(msg));
        }
        catch (exception& e)
        {
             cerr << e.what();
             exit 1;
        }
        
    Sources: http://www.gnu.org/s/libc/manual/html_node/Backtraces.html
             http://tombarta.wordpress.com/2008/08/01/c-stack-traces-with-gcc
             
    emmanuel.courcelle@toulouse.inra.fr
    
*/

#include <stdexcept>
#include <string>
using namespace std;

#define STRINGIFY(x) #x
#define TOSTRING(x) STRINGIFY(x)
#define ERROR_FILE_LINE(file,line) (string) "File: " + (string) file + (string) ":" + (string) STRINGIFY(line)
class rtstack: public runtime_error {
public:
	explicit rtstack(const string & __arg);
private:
	string __getStack() const throw();
};

#endif

