#! /usr/bin/perl

# $Id: testconvert.pl 648 2011-10-31 12:59:22Z manu $

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

use strict;
use warnings;
use File::Spec;
use File::Basename;

# Finding some perl modules distributed with glint 
use FindBin;
use lib "$FindBin::Bin/perl";
use General;
use Sufine;
use ParamParser;

use constant GLINT_OPT_DEF      => "";
use constant GLINTINDEX_OPT_DEF => "-C 0";
use constant TIME_OPT           => "-f '%E %S %U %K %M %F'";
use constant GLINT_EXE          => 'glint';
#use constant GLINT_EXE          => '/home/manu/devt-local/glint/src/glint';
use constant GLINTCONVERT_EXE     => 'glint';
#use constant GLINTCONVERT_EXE     => '/home/manu/devt-local/glint/src/glintconvert';

MAIN:
{
	ParamParser::SetDefaultBehaviour('exit_on_getopt_error');
	my $o_param=New ParamParser('GETOPTLONG',\&Usage,'run','data_directory=s','target_directory=s','bank=s','request=s',
	                                                 'pivot_format=s', 'output_format=s',
	                                                 'self','check_svn','auto_index','glintindex_opt=s','glint_opt=s','help');
	
	$o_param->AssertDirExists('data_directory');
	$o_param->AssertDefined('target_directory');

	my $self_flg       = $o_param -> IsDefined('self');
	my $auto_index_flg = $o_param -> IsDefined('auto_index');
	
	if ( $self_flg && $auto_index_flg)
	{
		die "ERROR - Only one of self or auto_index must be specified. TODO -> ARRANGER CELA\n";
	}
	
	$o_param->SetUnlessDefined('glint_opt',&GLINT_OPT_DEF);
	$o_param->SetUnlessDefined('glintindex_opt',&GLINTINDEX_OPT_DEF);
	$o_param->SetUnlessDefined('bank','bank.fasta');
	$o_param->SetUnlessDefined('request','request.fasta');
	$o_param->SetUnlessDefined('pivot_format','bamext');
	$o_param->AssertAllowedValue('pivot_format','^[bs]amext$');
	$o_param->SetUnlessDefined('output_format','1');
	
	my $ref_dir     = File::Spec->rel2abs($o_param->Get('reference_directory'));
	my $bank        = $o_param->Get('bank');
	my $data        = $o_param->Get('data');
	
	# computing and may be creating the target directory
	my $target_dir  = File::Spec->rel2abs($o_param->Get('target_directory'));
	if ( $o_param->IsDefined('check_svn') && $o_param->GetInteger('check_svn') == &TRUE)
	{
		my $rev = &CheckRelease();
		$target_dir = File::Spec->catfile($target_dir,$rev);
		warn "SVN revision = $rev\n";
	}
	
	if ( -e $target_dir && ! -d $target_dir )
	{
		die "ERROR - $target_dir should not exist OR should be a directory\n";
	}
	mkdir $target_dir if ( ! -e $target_dir);
	    
	# Running glint saving the index, then glintconvert
	&RunGlintCheck( $o_param, $target_dir);
}

=pod

=head2 Function Cat

 Usage:     &Cat($fh_out,$filename)
 Procedure: print every line of $filenmae to $fh_out
 Arg :      $fh_out   An opened file handle, defaults to stdout
            $filename A file name

=cut
sub Cat
{
	my ( $fh_out, $file ) = @_;
	$fh_out = *STDOUT unless defined $fh_out;
	
	my $fh_in = new IO::File $file;
	while (my $line = <$fh_in>)
	{
		print $fh_out $line;
	}
}

=pod

=head2 Function ComputePrefix

 Usage:     my $prefix = &ComputePrefix( $dir );
 Function:  From the directory passed by parameter, taking its name and its directory name,
            compute a prefix that will be used in filenames
 Arg :      $dir (optional)   The directory to use for computing the prefix
                              The current directory is used by default
 Return:    The prefix

=cut
sub ComputePrefix
{
    my $dir    = shift;
    $dir       = File::Spec->rel2abs(File::Spec->curdir()) unless (defined $dir);
    my $prefix = basename(dirname($dir)).'_'.basename($dir);
	return $prefix;
}

=pod

=head2 Function CheckRelease

 Usage:     &CheckRelease
 Function:  Does work only if the perl package SVN is installed (protected by eval however)
			Calls svn status on the .. directory, toknow if there is anything to commit
			Dies if there are files with status Modified, Added or Conflicted
			Else, calls svn info and returns the revision number
 Arg :      none
 Return:    Release number

=cut

sub CheckRelease
{
	eval "use SVN::Client";
	eval "use SVN::Wc:Status";
	
	my $svn = new SVN::Client();
	my (@a_conflicted,@a_added,@a_modified,@a_other);
	no warnings;
	my $status_func = sub {
		my ( $path, $status) = @_;
		#print "coucou $path " . $status->text_status() ."\n";
		push (@a_conflicted, $path), return if ( $status->text_status() == $SVN::Wc::Status::conflicted );
		push (@a_modified, $path), return if ( $status->text_status() == $SVN::Wc::Status::modified );
		push (@a_added, $path), return if ( $status->text_status() == $SVN::Wc::Status::added );

	};
	use warnings;
	$svn->status('..','WORKING',$status_func,1,0,0,0);
	
	# If there are some items with the status 'Added'/'Modified'/'conflicted', we give up
	my $flg_exit = &FALSE;
	foreach my $path (@a_conflicted)
	{
		print "C $path\n";
		$flg_exit = &TRUE;
	}
	foreach my $path (@a_modified)
	{
		print "M $path\n";
		$flg_exit = &TRUE;
	}
	foreach my $path (@a_added)
	{
		print "A $path\n";
		$flg_exit = &TRUE;
	}
	foreach my $path (@a_other)
	{
		print "$path\n";
	}
	if ($flg_exit==&TRUE)
	{
		die "ERROR - Please commit before calling the --check_svn switch";
	}
	
	my $rev;
	my $receiver = sub {
		                my( $path, $info, $pool ) = @_;
		                $rev = $info->rev;
					};
    $svn->info( '..', undef, 'WORKING', $receiver, 0 );
    return $rev;
};

=pod

=head2 Procedure RunGlintCheck

 Usage:     &RunGlintCheck($o_param, $target_dir);
 Function:  Run glint, ie: 1/ run glint with the switch --save-index, comparing the request against the bank, in requested output format
                           2/ run glint and glintconvert through a pipe, using the requested pivot format
                           2/ Both outputs should be the same
 Arg :      $o_param, $target_dir

=cut
sub RunGlintCheck
{
	my ($o_param,$target_dir) = @_;

	my $sav_dir    = File::Spec->rel2abs(File::Spec->curdir());
	my $data_dir    = File::Spec->rel2abs($o_param->Get('data_directory'));
	my $path_bank   = File::Spec->file_name_is_absolute( $o_param->Get('bank') ) ? 
						$o_param->Get('bank')
						:
						File::Spec->catfile($data_dir,$o_param->Get('bank'));
	my ($b1,$b2,$bank) = File::Spec->splitpath( $path_bank );
	
	my $path_request= File::Spec->file_name_is_absolute( $o_param->Get('request') ) ? 
						$o_param->Get('request')
						:
						File::Spec->catfile($data_dir,$o_param->Get('request'));
	my ($r1,$r2,$request) = File::Spec->splitpath( $path_request );
	$request = "--self" if ( $o_param->Get('self'));
    
	my $glint_exe      = File::Spec->file_name_is_absolute( &GLINT_EXE ) ?
						 &GLINT_EXE
						 :
						 File::Spec->catfile($sav_dir,&GLINT_EXE);
	-x $glint_exe      || die &GLINT_EXE . " not found or not executable";
	$glint_exe .= " align";
	
	my $glintconvert_exe = File::Spec->file_name_is_absolute( &GLINTCONVERT_EXE ) ?
						 &GLINTCONVERT_EXE
						 :
						 File::Spec->catfile($sav_dir,&GLINTCONVERT_EXE);
	-x $glintconvert_exe || die &GLINTCONVERT_EXE . " not found or not executable";
	$glintconvert_exe .= " convert";


	my $output_format  = $o_param->Get('output_format');
	my $pivot_format   = $o_param->Get('pivot_format');
	my $glint_opt      = $o_param->Get('glint_opt');

	chdir $target_dir;
	warn "Now working in directory $target_dir\n";

    # Create a symlink to the bank and request, avoiding a copy
	symlink ( $path_bank, $bank       );
	symlink ( $path_request, $request );	

    my $prefix    = &ComputePrefix();
    my $fh_out    = New Sufine(-name => ">${prefix}_out",-ext => 'out');
    my $file_out  = $fh_out->GetFileName();
    $fh_out -> close();
    
    my $fh_err   = New Sufine(-name => ">${prefix}_err",-ext => 'err');
    my $file_err = $fh_err->GetFileName();
    
    my $fh_cvt_out = New Sufine(-name => ">${prefix}_convert_out",-ext => 'out');
    my $file_cvt_out= $fh_cvt_out->GetFileName();
    $fh_cvt_out -> close();
    
    my ($cmd,$msg);
    # Executing glint
    $cmd = "$glint_exe $glint_opt --save-index -m $output_format $bank $request -o $file_out 2>>$file_err";
    $msg = "STARTING glint\n=====================\n$cmd\n";
    warn $msg;
    print $fh_err $msg;
	system("$cmd");
    
    # Executing glint + glintconvert
    print $fh_err "\n";
    $cmd = "$glint_exe $glint_opt -m $pivot_format $bank $request | $glintconvert_exe --input-format $pivot_format -m $output_format -o $file_cvt_out 2>>$file_err";
    $msg = "STARTING glint|glintconvert\n=====================\n$cmd\n";
	system("$cmd");
    warn $msg;
    print $fh_err $msg;
    
	# comparing the results
	my $diff = `diff $file_out $file_cvt_out`;
	
	if ( $diff eq "" )
	{
		print "Test OK\n";
	}
	else
	{
		print "Test failed\n";
		print $diff;
	}
    
    chdir $sav_dir;
}

sub Usage
{
    print "Calling glintindex, glint and glintconvert, comparing the results\n";
    print "=================================================================\n";
    print "testconvert.pl --run \n";
    print "               --data_directory      <data_directory>\n";
    print "               --target_directory    <output directory>\n";
    print "               --output_format       <cf. glint --help>\n";
    print "               --pivot_format samext/bamext (default bamext) read by glintconvert\n";
    print "               [--check_svn] add a directory tree, its name is the svn revision\n";
    print "               [--bank=...]     (relative to data directory, default=bank.fasta)\n";
    print "               [--auto_index]   using auto index mode: glintindex is not called\n";
    print "               [--request=...]  (relative to data directory, default=request.fasta)\n";
    print "               [--self]         using self mode, ignoring request)\n";
    print "               [--glint_opt     =<options for glint>]\n\n";
}
